/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2011 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.framework.engine.pipeline;

import java.io.Serializable;
import java.util.HashMap;

/**
 * This class allows at the elements of the pipeline to add their property
 * independently regarding the other elements.
 * <p>
 * Standard properties are:
 * <ul>
 *   funambol.sessionid - the server session id
 * </ul>
 * <p>
 * <b>NOTE:</b> the prefix <i>funambol.</i> for property keys is reserved for
 * Funambol internal use.
 *
 * @version $Id: MessageProcessingContext.java,v 1.2 2006/08/07 21:09:20 nichele Exp $
 */
public class MessageProcessingContext implements Serializable {

    // --------------------------------------------------------------- Constants

    public static final String PROPERTY_SESSIONID = "funambol.sessionid";

    // ------------------------------------------------------------ Private data

    private HashMap<String, Object> properties = new HashMap<String, Object>();

    // ------------------------------------------------------------ Constructors

    // -------------------------------------------------------------- Properties

    /**
     * Getter for properties.
     * @return Value of properties.
     */
    public HashMap<String, Object> getProperties() {
        return this.properties;
    }

    /**
     * Setter for properties.
     * @param properties New value of property.
     */
    public void setProperties(HashMap<String, Object> properties) {
        this.properties = properties;
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Returns the value of the property associated to the given key
     *
     * @param key the context property key
     *
     * @return the property value or null if key was not found
     */
    public Object getProperty(String key) {
        return properties.get(key);
    }

    /**
     * Sets a given context property
     *
     * @param key the context property key
     * @param value the context property value
     *
     */
    public void setProperty(String key, Object value) {
        properties.put(key, value);
    }

}
